@@ -1,5 +1,8 @@
 #![cfg_attr(not(test), no_std)]
 
+#[cfg(feature = "std")]
+extern crate std;
+
 mod client;
 mod eventloop;
 mod logging;
@@ -11,14 +14,12 @@ pub use bbqueue;
 
 pub use client::Client;
 use core::convert::TryFrom;
-use embedded_time::clock;
 pub use eventloop::EventLoop;
-use heapless::pool::{singleton::Box, Init};
 use heapless::{String, Vec};
 pub use mqttrust::encoding::v4::{Pid, Publish, QoS, QosPid, Suback};
 pub use mqttrust::*;
 pub use options::{Broker, MqttOptions};
-use state::{StateError, BOXED_PUBLISH};
+use state::StateError;
 
 #[derive(Debug, PartialEq)]
 #[cfg_attr(feature = "defmt-impl", derive(defmt::Format))]
@@ -27,7 +28,7 @@ pub struct PublishNotification {
     pub qospid: QoS,
     pub retain: bool,
     pub topic_name: String<256>,
-    pub payload: Vec<u8, 1024>,
+    pub payload: Vec<u8, 4096>,
 }
 
 /// Includes incoming packets from the network and other interesting events
@@ -38,15 +39,20 @@ pub enum Notification {
     /// Incoming connection acknowledge
     ConnAck,
     /// Incoming publish from the broker
-    Publish(Box<BOXED_PUBLISH, Init>),
+    #[cfg(not(feature = "std"))]
+    Publish(heapless::pool::singleton::Box<state::BoxedPublish, heapless::pool::Init>),
+    #[cfg(feature = "std")]
+    Publish(std::boxed::Box<PublishNotification>),
     /// Incoming puback from the broker
     Puback(Pid),
     /// Incoming pubrec from the broker
     Pubrec(Pid),
     /// Incoming pubcomp from the broker
     Pubcomp(Pid),
-    /// Incoming suback from the broker
+    // TODO:
     // Suback(Suback),
+    /// Incoming suback from the broker
+    Suback(Pid),
     /// Incoming unsuback from the broker
     Unsuback(Pid),
     // Eventloop error
@@ -105,9 +111,3 @@ impl From<StateError> for EventError {
         EventError::MqttState(e)
     }
 }
-
-impl From<clock::Error> for EventError {
-    fn from(_e: clock::Error) -> Self {
-        EventError::Clock
-    }
-}
